﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
//using System.Threading.Tasks; //.NET Framework 3.5 には含まれない
using System.Windows.Forms;

namespace Alarm
{
    public partial class Alarm : Form
    {
        public Alarm()
        {
            InitializeComponent();
        }

        // <Ver 1.10>アラーム時間の値(文字列)
        public static string m_alarmTime;

        // <Ver 1.11>アラーム解除フラグ
        public static bool m_alarmCanceled;

        // <Ver 1.13>前面表示時のアラーム画面サイズフラグ
        public static bool m_Norm_norm;

        // <Ver 1.13>アイコン表示時のアラーム画面サイズフラグ
        public static bool m_Icon_norm;

        // アラーム設定フォームの(再)描画
        private void Alarm_Load(object sender, EventArgs e)
        {
            // <Ver 1.10>アラーム時刻の読み込み
            if (m_alarmTime != null)
                setAlarm.Value = DateTime.Parse(m_alarmTime);
            else
                setAlarm.Value = DateTime.Now;

            // <Ver 1.11>アラーム解除フラグの読み込み
            if (m_alarmCanceled)
                checkbox_cancelAlarm.CheckState = CheckState.Checked;
            else
                checkbox_cancelAlarm.CheckState = CheckState.Unchecked;

            // <Ver 1.13>前面表示時のアラーム画面サイズフラグの読み込み
            if (m_Norm_norm)
            {
                Norm_normDisp.Text = "▶ 前面";
                m_Norm_norm = true;
            }
            else
            {
                Norm_normDisp.Text = "▶ 最大";
                m_Norm_norm = false;
            }

            // <Ver 1.13>アイコン表示時のアラーム画面サイズフラグの読み込み
            if (m_Icon_norm)
            {
                Icon_maxDisp.Text = "▶ 前面";
                m_Icon_norm = true;
            }
            else
            {
                Icon_maxDisp.Text = "▶ 最大";
                m_Icon_norm = false;
            }

            // <Ver 1.11>アラーム設定フォームを時計の周りに重ならないように表示する
            // <Ver 1.11>今 Alarmフォームがあるディスプレイの高さと幅を取得
            int screenH = System.Windows.Forms.Screen.GetBounds(this).Height;
            int screenW = System.Windows.Forms.Screen.GetBounds(this).Width;

            // <Ver 1.13> 単独起動ではOwnerが無いので異常終了するので、例外処理で終了
            try
            {

                // <Ver 1.11>MyClockフォームの左上の位置(left, top, width, height)を取得
                var left = this.Owner.Location.X;
                var top = this.Owner.Location.Y;
                var width = this.Owner.Width;
                var height = this.Owner.Height;

                // <Ver 1.11>Alarmフォームのデフォルトの位置(Dislayフォームの右隣)
                int X = left + width;
                int Y = top;

                // <Ver 1.11>Alarmフォームがスクリーンからはみ出す時、位置(X, Y)を更新
                if (X + this.Width > screenW)
                    X = left - this.Width;
                if (Y + this.Height > screenH)
                    Y = top + height - this.Height;

                // <Ver 1.11>親ウィンドウ (MyClock)の周りに移動させる
                this.Location = new Point(X, Y);

            }
            catch (NullReferenceException)
            {
                MessageBox.Show("単独では起動できません.\nMyClockから呼び出して使います.",
                                    "Alarm.exe", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                Dispose();
            }
        }

        // <Ver 1.10>[OK]ボタンクリックされた時のイベントハンドラ
        // DateTimePickerの値を更新して終了する
        private void button_OK_Click(object sender, EventArgs e)
        {
            m_alarmTime = setAlarm.Value.ToString("HH:mm:ss");
            // <Ver 1.10>[OK]クリックでもモーダルダイアログが OK を返さないので、
            // <Ver 1.10>強制的にOKを返す
            this.DialogResult = DialogResult.OK;
            Dispose();
        }

        // <Ver 1.10>現在時刻の取得ボタンのイベントハンドラ
        // <Ve 1.10>setAlarmを現在時刻で更新し、[OK]ボタンをアクティブにする
        private void button_curTime_Click(object sender, EventArgs e)
        {
            setAlarm.Value = DateTime.Now;
            button_OK.Select();
        }

        // <Ver 1.11>チェックボックスのイベントハンドラ
        // <Ver 1.11>アラーム解除チェックボックスの操作
        private void cancelAlarm_CheckedChanged(object sender, EventArgs e)
        {
            switch (checkbox_cancelAlarm.CheckState)
            {
                case CheckState.Checked:
                    m_alarmCanceled = true;
                    break;
                case CheckState.Unchecked:
                case CheckState.Indeterminate:
                    m_alarmCanceled = false;
                    break;
            }
        }

        private void Norm_normDisp_Click(object sender, EventArgs e)
        {
            if (m_Norm_norm)
            {
                Norm_normDisp.Text = "▶ 最大";
                m_Norm_norm = false;
            }
            else
            {
                Norm_normDisp.Text = "▶ 前面";
                m_Norm_norm = true;
            }
            button_OK.Select();
        }

        private void Icon_maxDisp_Click(object sender, EventArgs e)
        {
            if (m_Icon_norm)
            {
                Icon_maxDisp.Text = "▶ 最大";
                m_Icon_norm = false;
            }
            else
            {
                Icon_maxDisp.Text = "▶ 前面";
                m_Icon_norm = true;
            }
            button_OK.Select();
        }
    }
}
