﻿using Alarm;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
//using System.Threading.Tasks; // .NET Franework 3.5 では使えないのでコメントアウト
using System.Windows.Forms;
using System.Runtime.InteropServices;   // DllImportのために追加
using System.Diagnostics; // ウィンドウプロセス取得のために追加

namespace MyClock
{
    public partial class Display : Form
    {
        public Display()
        {
            InitializeComponent();

            // <Ver 1.00>マウスのメッセーハンドラ作成
            currentTime.MouseDown += new MouseEventHandler(Display_MouseDown);
            currentTime.MouseMove += new MouseEventHandler(Display_MouseMove);

            //<ver 1.12>設定ファイル (MyClock.exe.config) からの読み込み
            Properties.Settings.Default.Reload();
            m_Alarm = Properties.Settings.Default.alarm_Time;   // アラーム設定時刻
            m_cancelAlarm = Properties.Settings.Default.alarm_Canceled; //アラーム解除設定
            m_main_left = Properties.Settings.Default.main_left;    //時計表示位置-左
            m_main_top = Properties.Settings.Default.main_top;  //時計表示位置-上
            m_size_normal = Properties.Settings.Default.main_size_normal;   //時計サイズ
            m_Norm_norm = Properties.Settings.Default.main_norm_norm;   //前面表示⇒前面アラーム
            m_Icon_norm = Properties.Settings.Default.main_icon_norm;   //アイコン⇒前面アラーム
        }

        private static int count;

        //<Ver1.12>時計の位置の格納変数
        private static int m_main_top;
        private static int m_main_left;

        //
        // <Ver 1.13>時計フォームの位置を取得する
        //
        private void getWinPos()
        {
            m_main_left = this.Left;
            m_main_top = this.Top;
        }

        //<Ver1.13>時計サイズの格納変数
        private static bool m_size_normal;

        //  Form描画のメッセージハンドラ
        private void Display_Load(object sender, EventArgs e)
        {
            DisplayTime();  // <Ver 1.00>起動時タイマー動作までのの時刻表示
            this.TopMost = true;    // <Ver 1.11>フォーム描画時に再度前面表示を指定

            // <Ver 1.13>前回終了時に最小化されていた時は最小化する
            if (!m_size_normal)
            {
                this.WindowState = FormWindowState.Minimized;
                m_size_normal = true;
            }

            // <Ver 1.12>時計が終了した時の位置に復元する
            this.Location = new Point(m_main_left, m_main_top);
        }

        // <Ver 1.00>Timerのメッセージハンドラ
        private void secTimer_Tick(object sender, EventArgs e)
        {
            DisplayTime();
            if (this.WindowState == FormWindowState.Normal)
            {
                m_size_normal = true;
            }
            else if (this.WindowState == FormWindowState.Minimized)
            {
                m_size_normal = false;
            }
        }

        // <Ver 1.14>アラーム発動中フラグ
        private static bool m_alarm_mode = false;

        // <Ver 1.00>現在時刻の表示
        private void DisplayTime()
        {
            string time = DateTime.Now.ToString("HH:mm:ss");
            currentTime.Text = time;

            // <Ver 1.10>アラーム設定時間になったら時計の外観を変えて点滅させる
            if (time == m_Alarm)
            {
                if (!m_cancelAlarm)
                {
                    m_alarm_mode = true;    //<Ver 1.14>アラーム発動フラグを立てる
                    //<Ver 1.13>設定ファイルに保存する現在の時計サイズを取得する
                    if (this.WindowState == FormWindowState.Normal)
                        m_display_norm = true;
                    else if (this.WindowState==FormWindowState.Minimized)
                        m_display_norm = false;

                    // <Ver 1.10>アラーム発動時のフォーム形状を変更し点滅させる
                    ChangeDisplay();
                    FlashWnd();
                }
            }
        }

        // <Ver 1.10>ラベルをシングルクリックした時のイベントハンドラ
        // <Ver 1.14>フォーム形状を元に戻すのは、アラーム発動フラグが下りている時のみに制限
        private void currentTime_Click(object sender, EventArgs e)
        {
            // <Ver 1.14>フォームを元に戻すのは、
            // <Ver 1.14>アラーム発動フラグが立っている時のみに変更(機能追加のため)
            if (m_alarm_mode == true)
            {
                // <Ver 1.10>フォーム形状を元に戻し、点滅を止める
                ResumeDisplay();
                StopFlashWnd();
                // <Ver 1.14>アラーム発動フラグを下ろす
                m_alarm_mode = false; 
            }
        }

        // <Ver 1.00>時刻表示ラベル上でのマウス左ボタンダブルクリックのメッセーハンドラ
        private void currentTime_DoubleClick(object sender, EventArgs e)
        {
            getWinPos();    //<Ver 1.13>時計の位置を取得する

            //プログラムを終了する
            // <Ver1.14>最小化で終了した時にフォーム状態を設定ファイルに保存するために
            // <Ver 1.14> Application.Exit() 実行後 FormClosedイベントハンドラで処理する
            //Dispose(); // <Ver 1.14>Dispose()では FormClosedを捕らえられない
            Application.Exit(); 
        }

        // <Ver 1.10>設定ファイルに保存するために、マウスのクリック位置を取得する変数
        private Point mousePoint;

        // <Ver 1.00>DisplayのMouseDownイベントハンドラ
        // <Ver 1.10>マウスのボタンが押されているときの処理
        private void Display_MouseDown(object sender, MouseEventArgs e)
        {
            // <Ver 1.10>クリックが左ボタンの場合は、マウス位置を取得しカーソルをHandに変更する
            if ((e.Button & MouseButtons.Left) == MouseButtons.Left)
            {
                //カーソルをHandに変更
                this.Cursor = Cursors.Hand;
                //位置を記憶する
                mousePoint = new Point(e.X, e.Y);
            }
            else
            {
                //カーソルを元に戻す
                this.Cursor = Cursors.Default;
            }

            // <Ver 1.10>クリックが右ボタンの場合は、時計を最小化する
            if ((e.Button & MouseButtons.Right) == MouseButtons.Right)
            {
                getWinPos();    //<Ver 1.13>時計の位置を取得する
                //最小化する
                this.WindowState = FormWindowState.Minimized;
            }
        }

        // <Ver 1.00>DisplayのMouseMoveイベントハンドラ
        // <Ver 1.00>マウスが動いたときの処理
        private void Display_MouseMove(object sender, MouseEventArgs e)
        {
            if ((e.Button & MouseButtons.Left) == MouseButtons.Left)
            {
                //フォームの位置変更
                this.Left += e.X - mousePoint.X;
                this.Top += e.Y - mousePoint.Y;
            }
            else
            {
                //カーソルを元に戻す
                this.Cursor = Cursors.Default;
            }
        }

        // <Ver 1.10>アラーム時刻の格納変数
        private static string m_Alarm;
        // <Ver 1.11>アラーム解除フラグの格納変数
        private static bool m_cancelAlarm = false;
        // <Ver1.14>アラーム発動前の時計サイズ
        private static bool m_display_norm;
        // <ver 1.14>アラーム画面設定
        private static bool m_Norm_norm;
        private static bool m_Icon_norm;

        // <Ver 1.14.1>アラーム設定処理
        private void setAlarm()
        {
            // <Ver 1.14>前面表示時のアラーム画面サイズフラグの更新
            Alarm.Alarm.m_Norm_norm = m_Norm_norm;
            // <Ver 1.14>アイコン表示時のアラーム画面サイズフラグの更新
            Alarm.Alarm.m_Icon_norm = m_Icon_norm;
            // <Ver 1.11>アラーム解除フラグの更新
            Alarm.Alarm.m_alarmCanceled = m_cancelAlarm;
            // <Ver 1.10>m_Alarmが値を格納している時は、それでsetAlatmの値を更新する
            if (m_Alarm != null)
                Alarm.Alarm.m_alarmTime = m_Alarm;

            // <Ver 1.10>アラーム設定フォームのインスタンス作成
            Alarm.Alarm dlg = new Alarm.Alarm();
            // <Ver 1.10>アラーム設定フォームをモーダルダイアログで表示
            if (dlg.ShowDialog(this) == DialogResult.OK)
            {
                // OK が返ってきたときの処理
                // <Version 1.13>アラーム画面設定
                m_Norm_norm = Alarm.Alarm.m_Norm_norm;
                m_Icon_norm = Alarm.Alarm.m_Icon_norm;
                // <Ver1.11>アラーム解除フラグ
                m_cancelAlarm = Alarm.Alarm.m_alarmCanceled;
                // <Ver 1.10>アラーム設定時刻
                string m_time = Alarm.Alarm.m_alarmTime;
                if (m_time != null || m_time != m_Alarm)
                {
                    // アラーム設定時刻が変更された場合、static m_Alarm 変数を更新
                    m_Alarm = Alarm.Alarm.m_alarmTime;
                }
            }

            // MSDNでダイアログの破棄をすべきと推奨している
            dlg.Dispose();
        }

        // <Ver 1.14.2>アセンブリ情報の「タイトル」を返す
        private string getAssemblyTitle()
        {
            System.Reflection.AssemblyTitleAttribute title =
                (System.Reflection.AssemblyTitleAttribute)Attribute.GetCustomAttribute(
                    System.Reflection.Assembly.GetExecutingAssembly(),
                    typeof(System.Reflection.AssemblyTitleAttribute));

            return title.Title.ToString();
        }
        
        // <Ver 1.14.2>アセンブリ情報の「製品」を返す
        private string getAssemblyProduct()
        {
            System.Reflection.AssemblyProductAttribute product=
                (System.Reflection.AssemblyProductAttribute)Attribute.GetCustomAttribute(
                    System.Reflection.Assembly.GetExecutingAssembly(),
                    typeof(System.Reflection.AssemblyProductAttribute));

            return product.Product.ToString();
        }

        // <Ver 1.14.2>アセンブリ情報の「バージョン」を返す
        private string getAssemblyVersion()
        {
            //自分自身のAssembly情報を取得
            System.Reflection.Assembly assembly =
                System.Reflection.Assembly.GetExecutingAssembly();
            //バージョンの取得
            System.Version ver = assembly.GetName().Version;

            return ver.ToString();
        }

        // <Ver 1.14.2>アセンブリ情報の「バージョン」を返す
        private string getAssemblyCopyright()
        {
            System.Reflection.AssemblyCopyrightAttribute copyright =
                (System.Reflection.AssemblyCopyrightAttribute)Attribute.GetCustomAttribute(
                    System.Reflection.Assembly.GetExecutingAssembly(),
                    typeof(System.Reflection.AssemblyCopyrightAttribute));

            return copyright.Copyright.ToString();
        }

        // <Ver 1.14.2>コントロールパネル設定処理
        private void setCtrlPanel()
        {
            // 取り敢えず製品情報を表示する⇒今後の機能拡張設定はここから行う
            string msg = getAssemblyTitle();
            msg += "\n\n　";
            msg += getAssemblyProduct() +" Ver. " + getAssemblyVersion()+"\n　";
            msg += getAssemblyCopyright() + "\n\n";
            msg += "　【使い方】\n";
            msg += "　　時計をクリックしてから...\n";
            msg += "　　・[Esc]キーでアラーム機能を設定\n";
            msg += "　　・[F1]キーでこのパネルを表示\n\n";
            MessageBox.Show(msg, getAssemblyProduct()+"について...", 
                MessageBoxButtons.OK, MessageBoxIcon.Question);
        }

        // <Ver 1.10>キーを押した時のイベントハンドラ
        private void Display_KeyDown(object sender, KeyEventArgs e)
        {
            // アラーム動作確認用
            /*if (e.KeyCode == Keys.Tab)
            {
                ChangeDisplay();
                FlashWnd();
            }*/
            //動作確認用ここまで

            if (e.KeyCode == Keys.Escape)
                // <Ver 1.10>[Esc]キーを押した時にアラーム設定
                // <Ver 1.14.1>処理をまるごと setAlaem()にコピー
                setAlarm();

            else if (e.KeyCode == Keys.F1)
                // <Ver 1.14.1>[F1]キーを押した時にコントロールパネル設定
                setCtrlPanel();
        }

        // <Ver 1.10>Win32 APIを使うための準備
        [DllImport("user32.dll")]
        static extern Int32 FlashWindowEx(ref FLASHWINFO pwfi);

        // <Ver 1.10>FLASHWININFO 構造体のオーバーライド
        [StructLayout(LayoutKind.Sequential)]
        public struct FLASHWINFO
        {
            public UInt32 cbSize;           // FLASHWINFO構造体のサイズ
            public IntPtr hwnd;              // 点滅対象のウィンドウ・ハンドル
            public UInt32 dwFlags;        // 以下の「FLASHW_XXX」のいずれか
            public UInt32 uCount;         // 点滅する回数
            public UInt32 dwTimeout;   // 点滅する間隔（ミリ秒単位）
        }

        // <Ver 1.10>FLASHWININFO メンバの実装
        // 点滅を止める
            public const UInt32 FLASHW_STOP = 0;
        // タイトルバーを点滅させる
            public const UInt32 FLASHW_CAPTION = 1;
        // タスクバー・ボタンを点滅させる
            public const UInt32 FLASHW_TRAY = 2;
        // タスクバー・ボタンとタイトルバーを点滅させる
            public const UInt32 FLASHW_ALL = 3;
        // FLASHW_STOPが指定されるまでずっと点滅させる
            public const UInt32 FLASHW_TIMER = 4;
        // ウィンドウが最前面に来るまでずっと点滅させる
            public const UInt32 FLASHW_TIMERNOFG = 12;

        // <Ver 1.10>時計の見え方を変える
        // <Ver 1.14>アラーム発動時の時計の形状を選択できるようにしたのでロジック変更
        private void ChangeDisplay()
        {
            // ウィンドウスタイルを変更する
            this.FormBorderStyle = FormBorderStyle.FixedToolWindow;
            if (m_display_norm)
            {
                if (m_Norm_norm)
                {
                    this.WindowState = FormWindowState.Normal;
                    this.Opacity = 1D;
                    this.Text = "It's Time! - Click Here";
                }
                else
                {
                    this.WindowState = FormWindowState.Maximized;
                    this.Opacity = 0.5D;
                }
            }
            else
            {
                if (m_Icon_norm)
                {
                    this.WindowState = FormWindowState.Normal;
                    this.Opacity = 1D;
                    this.Text = "It's Time! - Click Here";
                }
                else
                {
                    this.WindowState = FormWindowState.Maximized;
                    this.Opacity = 0.5D;
                }
            }
        }

        // <Ver 1.10>時計の見え方を元に戻す
        private void ResumeDisplay()
        {
            this.FormBorderStyle = FormBorderStyle.None;
            this.WindowState = FormWindowState.Normal;
            this.Opacity = 0.5D;
            this.Text = "MyClock";
            //<Ver 1.14>復元時はノーマルスタイルが使いやすそうなので以下はコメントアウト
            //if (!m_display_norm)
            //    this.WindowState = FormWindowState.Minimized;
        }

        // <Ver 1.10>ウィンドウを点滅させる
        private void FlashWnd()
        {
            FLASHWINFO flsh = new FLASHWINFO();
            flsh.cbSize = Convert.ToUInt32(Marshal.SizeOf(flsh));
            flsh.hwnd = this.Handle;
            flsh.dwFlags = FLASHW_ALL | FLASHW_TIMER;
            //flsh.uCount = 5;
            flsh.dwTimeout = 100;

            FlashWindowEx(ref flsh);
        }

        // <Ver 1.10>ウィンドウの点滅を止める
        private void StopFlashWnd()
        {
            FLASHWINFO flsh = new FLASHWINFO();
            flsh.cbSize = Convert.ToUInt32(Marshal.SizeOf(flsh));
            flsh.hwnd = this.Handle;
            flsh.dwFlags = FLASHW_STOP;

            FlashWindowEx(ref flsh);
        }

        // <Ver 1.13>終了時にアプリ設定を保存する (MyClock.exe.config)
        // <Ver 1.14>設定保存を FormClosed()で行うように変更した
        private void Display_FormClosed(object sender, FormClosedEventArgs e)
        {
            //<Ver1.13>時計サイズの保存を行うため、DoubleClickイベントハンドラから移動
            //getWinPos(); //DoubleClickイベントハンドラで、時計フォームの位置を取得しておく
            Properties.Settings.Default.alarm_Time = m_Alarm;   // アラーム設定時刻
            Properties.Settings.Default.alarm_Canceled = m_cancelAlarm; // アラーム解除設定
            Properties.Settings.Default.main_left = m_main_left;    // 時計の位置-左
            Properties.Settings.Default.main_top = m_main_top;  // 時計の位置-上
            Properties.Settings.Default.main_size_normal = m_size_normal;   // 時計のサイズ
            Properties.Settings.Default.main_norm_norm = m_Norm_norm;   //前面表示⇒前面アラーム
            Properties.Settings.Default.main_icon_norm = m_Icon_norm;   //アイコン⇒前面アラーム
            Properties.Settings.Default.Save();
        }
    }
}
